@@ -1,6 +1,9 @@
+#undef PLATFORM
 /* Define if you have the unicon library libimm_server and libimmclient.a installed).  */
 #undef HAVE_UNICON_LIB
 /* Define if you have the unicon library libimm_server and libimmclient.a installed).  */
 #undef HAVE_GGI_LIB
 #undef PREFIX
+/* Define if you compile on a x86 machine.  */
+#undef USING_VGA
 
