@@ -54,6 +54,11 @@ mEscState(NORMAL)
     memset(mpHistAttr, mAttr, size);
     memset(mpHistFlag, txtASCII | txtUpdated, size);
     mHistCurRow = 0;
+
+    mMouseMask = 0x77;  // 0x80 when mono
+    mMouseIdx = -1;
+    mSelStart = -1;  // cleared by clear_selection
+    mpSelBuf = NULL;
 }
 
 Console::~Console() {
@@ -63,6 +68,7 @@ Console::~Console() {
     delete[] mpSavTextBuf;
     delete[] mpSavAttrBuf;
     delete[] mpSavFlagBuf;
+    delete[] mpSelBuf;
 }
 
 /** handle ansi esc command */
@@ -78,11 +84,11 @@ void Console::DoEscape(char c) {
                 Cr();
                 Lf();
                 break;
-            case 'H':                  //Set Tab <ESC>H
+            case 'H':  // Set Tab <ESC>H
                 mTabStop[Row() >> 5] |= (1 << (Row() & 31));
                 break;
-                //Reverse Index,Scroll Up
-            case 'M':
+            case 'M':  // Reverse Index,Scroll Up
+            case 'T':  // FreeBSD, cons25
                 if (Row() == mScrollStart)
                     ScrollScr(DOWN);
                 else if (Row())
@@ -99,12 +105,10 @@ void Console::DoEscape(char c) {
             case '=':
                 assert(!"=");
                 break;
-                //Save Cursor & Attrs <ESC>7
-            case '7':
+            case '7':  // Save Cursor & Attrs <ESC>7
                 SaveCursor();
                 break;
-                //Restore Cursor & Attrs  <ESC>8
-            case '8':
+            case '8':  // Restore Cursor & Attrs  <ESC>8
                 UnSaveCursor();
                 break;
             default:
@@ -117,28 +121,24 @@ void Console::DoEscape(char c) {
                                        //param count
         num = mpEscParam - mEscParam + 1;
         switch (c) {
-            //Restore Cursor & Attrs  <ESC>8
-            case 'A':
+            case 'A':  // Restore Cursor & Attrs  <ESC>8
                 if (!mEscParam[0])
                     mEscParam[0]++;
                 ConGoto(Col(), Row() - mEscParam[0]);
                 break;
-                //Cursor Down <ESC>[{COUNT}B
-            case 'B':
+            case 'B':  // Cursor Down <ESC>[{COUNT}B
             case 'e':
                 if (!mEscParam[0])
                     mEscParam[0]++;
                 ConGoto(Col(), Row() + mEscParam[0]);
                 break;
-                //Cursor Forward <ESC>[{COUNT}C
-            case 'C':
+            case 'C':  // Cursor Forward <ESC>[{COUNT}C
             case 'a':
                 if (!mEscParam[0])
                     mEscParam[0]++;
                 ConGoto(Col() + mEscParam[0], Row());
                 break;
-                //Cursor Backward <ESC>[{COUNT}D
-            case 'D':
+            case 'D':  // Cursor Backward <ESC>[{COUNT}D
                 if (!mEscParam[0])
                     mEscParam[0]++;
                 ConGoto(Col() - mEscParam[0], Row());
@@ -153,15 +153,13 @@ void Console::DoEscape(char c) {
                     mEscParam[0]++;
                 ConGoto(0, Row() - mEscParam[0]);
                 break;
-                //Cursor Horizontal Absolute
-            case 'G':
+            case 'G':  // Cursor Horizontal Absolute
             case '`':
                 if (mEscParam[0])
                     mEscParam[0]--;
                 ConGoto(mEscParam[0], Row());
                 break;
-                //ESC [m;nH moves cursor to (m,n)
-            case 'H':
+            case 'H':  // ESC [m;nH moves cursor to (m,n)
             case 'f':
                 if (mEscParam[0])
                     mEscParam[0]--;
@@ -169,77 +167,64 @@ void Console::DoEscape(char c) {
                     mEscParam[1]--;
                 AbsGoto(mEscParam[1], mEscParam[0]);
                 break;
-                //ESC [sJ clears in display
-            case 'J':
+            case 'J':  // ESC [sJ clears in display
                 switch (mEscParam[0]) {
-                    case 0:
-                        //clear from cursor to end of screen
+                    case 0:  // clear from cursor to end of screen
                         Clear(Col(), Row(), mConEndCol, Row(), mAttr);
                         if (Row() < mConEndRow)
                             Clear(0, Row() + 1, mConEndCol, mConEndRow, mAttr);
                         break;
-                    case 1:
-                        //clear from start to cursor
+                    case 1:  // clear from start to cursor
                         Clear(0, Row(), Col(), Row(), mAttr);
                         if (Row() > 0)
                             Clear(0, 0, mConEndCol, Row() - 1, mAttr);
                         break;
-                    case 2:
-                        //clear whole screen
+                    case 2:  // clear whole screen
                         Clear(0, 0, mConEndCol, mConEndRow, mAttr);
                         break;
-                    default:
-                        //do nothing
+                    default:  // do nothing
                         break;
                 }
                 mNeedWrap = false;
                 break;
-            case 'K':
-                //ESC [sK clears lines from cursor
+            case 'K':  // ESC [sK clears lines from cursor
                 switch (mEscParam[0]) {
-                    case 0:
-                        //clear from cursor to end of line
+                    case 0:  // clear from cursor to end of line
                         Clear(Col(), Row(), mConEndCol, Row(), mAttr);
                         break;
-                    case 1:
-                        //clear from beinning of line to cursor
+                    case 1:  // clear from beinning of line to cursor
                         Clear(0, Row(), Col(), Row(), mAttr);
                         break;
-                    case 2:
-                        //clear whole line
+                    case 2:  // clear whole line
                         Clear(0, Row(), mConEndCol, Row(), mAttr);
                         break;
-                    default:
-                        //do nothing
+                    default:  // do nothing
                         break;
                 }
                 mNeedWrap = false;
                 break;
-            case 'L':
-                //ESC [nL inserts n lines at cursor
+            case 'L':  // ESC [nL inserts n lines at cursor
                 InsertLine(mEscParam[0]);
                 break;
-            case 'M':
-                //ESC [nM deletes n lines at cursor
+            case 'M':  // ESC [nM deletes n lines at cursor
                 DeleteLine(mEscParam[0]);
                 break;
-            case 'P':
-                //ESC [nP deletes n chars at cursor
+            case 'P':  // ESC [nP deletes n chars at cursor
                 DeleteChar(mEscParam[0]);
                 break;
-            case 'X': {
-                int n = mEscParam[0];
-                if (!n)
-                    n++;
-                Clear(Col(), Row(),
-                      Col() + n > mConEndCol ? mConEndCol : Col() + n,
-                      Row(),
-                      mAttr);
-                mNeedWrap = false;
-            }
-            break;
-            case '@':
-                //ESC [n@ insert n chars at cursor
+            case 'X':
+                {
+                    int n = mEscParam[0];
+                    if (!n)
+                        n++;
+                    Clear(Col(), Row(),
+                          Col() + n > mConEndCol ? mConEndCol : Col() + n,
+                          Row(),
+                          mAttr);
+                    mNeedWrap = false;
+                }
+                break;
+            case '@':  // ESC [n@ insert n chars at cursor
                 InsertBlank(mEscParam[0]);
                 break;
             case ']':
@@ -258,8 +243,7 @@ void Console::DoEscape(char c) {
                     mEscParam[0]--;
                 AbsGoto(Col(), mEscParam[0]);
                 break;
-            case 'g':
-                //Clear Tab  <ESC>[{3}g
+            case 'g':  // Clear Tab  <ESC>[{3}g
                 if (num == 0)
                     mTabStop[Col() >> 5] &= ~(1 << (Col() & 31));
                 else if (mEscParam[0] == 3)
@@ -271,8 +255,7 @@ void Console::DoEscape(char c) {
             case 'l':
                 SetMode(false);
                 break;
-            case 'm':
-                //ESC [nm set attribute
+            case 'm':  // ESC [nm set attribute
                 for (int i = 0; i < num; i++) {
                     int value = mEscParam[i];
                     switch (value) {
@@ -285,20 +268,16 @@ void Console::DoEscape(char c) {
                         case 2:
                             mIntensity = 0;
                             break;
-                        case 4:
-                            //underline
+                        case 4:  // underline
                             mUnderline = true;
                             break;
-                        case 5:
-                            //blinking
+                        case 5:  // blinking
                             mBlink = true;
                             break;
-                        case 7:
-                            //reverse
+                        case 7:  // reverse
                             mReverse = true;
                             break;
-                        case 8:
-                            //hidden
+                        case 8:  // hidden
                             break;
                         case 10:
                             /* ANSI X3.64-1979 (SCO-ish?)
@@ -371,19 +350,16 @@ void Console::DoEscape(char c) {
                 }//for
                 UpdateAttr();
                 break;
-            case 'n':
-                //not implemented
+            case 'n':  // not implemented
                 assert(!"n");
                 break;
-            case 'p':
-                //Define Key
+            case 'p':  // Define Key
                 assert(!"Define Key not implemented yet!");
                 break;
             case 'q':
                 assert(!"esc q");
                 break;
-            case 'r':
-                //set scrolling range
+            case 'r':  // set scrolling range
                 if (!mEscParam[0])
                     mEscParam[0]++;
                 if (!mEscParam[1])
@@ -396,12 +372,10 @@ void Console::DoEscape(char c) {
                     AbsGoto(0, 0);
                 }
                 break;
-            case 's':
-                //Save Cursor <ESC>[s
+            case 's':  // Save Cursor <ESC>[s
                 SaveCursor();
                 break;
-            case 'u':
-                //Unsave Cursor <ESC>[u
+            case 'u':  // Unsave Cursor <ESC>[u
                 UnSaveCursor();
         } //switch
     }
@@ -420,8 +394,7 @@ void Console::ParseEscape(char c) {
             for (int i = 0; i < MAX_ESC_PARAMS; i++)
                 mEscParam[i] = 0;
             switch (c) {
-                case '#':
-                    //DEC obsolete hacks
+                case '#':  // DEC obsolete hacks
                     //anyone want to implement them?
                     break;
                 case '[':              //control sequence introducer
@@ -432,8 +405,8 @@ void Console::ParseEscape(char c) {
                     mEscIntro = c;
                     mEscState = NONSTD;
                     break;
-                case '%':
-                    //unfinished
+                case '%':  // unfinished
+
                     break;
                 case '(':              //Char Set G0
                 case ')':              //Char Set G1
@@ -508,6 +481,7 @@ void Console::SendChar(char c) {
 //#include <unistd.h> //for debug
 /** Write num chars in pBuf to console */
 void Console::Write(const char *pBuf, int num) {
+    SelClear(); // hide mouse
     bool isCursorOn = CursorOnOff();
     // turn off cursor for rapid display
     if (isCursorOn) CursorHide();
@@ -515,6 +489,7 @@ void Console::Write(const char *pBuf, int num) {
     char c;
     for (; num; pBuf++, num--) {
         c = *pBuf;
+        //debug<<c<<flush;
         if (mEscState != NORMAL)
             ParseEscape(c);
         else if (c < ' ')
@@ -846,11 +821,11 @@ void Console::ConGoto(int c, int r) {
 }
 
 //for absolute user moves, when decom is set
-inline void Console::AbsGoto(int c, int r) {
+void Console::AbsGoto(int c, int r) {
     ConGoto(c, mDecom ? (mScrollStart + r) : r);
 }
 
-inline void Console::Lf() {
+void Console::Lf() {
     /* don't scroll if above bottom of scrolling region, or
      * if below scrolling region
      */
@@ -861,7 +836,7 @@ inline void Console::Lf() {
     mNeedWrap = false;
 }
 
-inline void Console::Cr() {
+void Console::Cr() {
     Goto(0, Row());
     mNeedWrap = false;
 }
@@ -877,6 +852,7 @@ void Console::PushHistoryLine() {
 
 //scroll into history buffer and switch to history mode if necessary
 void Console::ScrollDelta(ScrollFlag f) {
+    SelClear();
     static int sBackOffset = 0;
     if (!mHistMode) {
         //history mode will be closed in PutChar()
@@ -984,6 +960,7 @@ Encode Console::DetectBufferEncode() {
 }
 
 void Console::VtSizeDelta(int ColDelta, int RowDelta) {
+    SelClear();
     int NewRows, NewEndRow;
     NewRows = mMaxRows - RowDelta;
     NewEndRow = NewRows - 1;
@@ -1022,3 +999,279 @@ void Console::UpdateAttr(){
         mAttr |= 0x80;
 }
 
+/* use complementary color to show the mouse pointer */
+void Console::SelPointer(const int offset) {
+    if (mMouseIdx >= 0 && mMouseIdx < mMaxCols * mMaxRows) {
+        mpAttr[mMouseIdx] ^= mMouseMask;
+        mpFlag[mMouseIdx] |= txtUpdated;
+
+        // Redraw();
+        RedrawChar(mMouseIdx % mMaxCols, mMouseIdx / mMaxCols);
+    }
+
+    if (offset < 0) {
+        mMouseIdx = -1;
+        return;
+    }
+
+    mMouseIdx = offset;
+    if (mMouseIdx > mMaxCols * mMaxRows - 1) {
+        mMouseIdx = mMaxCols * mMaxRows - 1;
+    }
+    
+    mpAttr[mMouseIdx] ^= mMouseMask;
+    mpFlag[mMouseIdx] |= txtUpdated;
+
+    // Redraw();
+    RedrawChar(mMouseIdx % mMaxCols, mMouseIdx / mMaxCols);
+}
+
+/* set reverse video on characters s-e of console with selection. */
+void Console::SelHighlight(const int begin, const int end) {
+    int count = end - begin + 1;
+    char* pAttr = &mpAttr[begin];
+    char* pFlag = &mpFlag[begin];
+    
+    char color;
+    while (count--) {
+         color = *pAttr;
+         //color = ((color) & 0x11) | (((color) & 0xe0) >> 4) | (((color) & 0x0e) << 4);
+         color = ((color) & 0x88) | (((color) & 0x70) >> 4) | (((color) & 0x07) << 4);
+         *pAttr = color;
+         pAttr++;
+         (*pFlag) |= txtUpdated;
+         pFlag++;
+         
+    }
+
+    // turn off cursor for rapid display
+    bool isCursorOn = CursorOnOff();
+    if (isCursorOn) CursorHide();
+    Redraw();
+    if (isCursorOn) CursorShow();
+    /*
+    int count = end - beign + 2;
+    unsigned short *p;
+
+    count /= 2;
+    p = screenpos(currcons, beign, viewed);
+    u16 *q = p;
+    int cnt = count;
+
+    if (!can_do_color) {
+        while (cnt--) *q++ ^= 0x0800;
+    } else if (hi_font_mask == 0x100) {
+        while (cnt--) {
+            u16 a = *q;
+            a = ((a) & 0x11ff) | (((a) & 0xe000) >> 4) | (((a) & 0x0e00) << 4);
+            *q++ = a;
+        }
+    } else {
+        while (cnt--) {
+            u16 a = *q;
+            a = ((a) & 0x88ff) | (((a) & 0x7000) >> 4) | (((a) & 0x0700) << 4);
+            *q++ = a;
+        }
+    }
+    if (DO_UPDATE)
+        do_update_region(currcons, (unsigned long) p, count);
+    */
+}
+
+/* remove the current selection highlight, if any,
+   from the console holding the selection. */
+void Console::SelClear() {
+    SelPointer(-1); /* hide the pointer */
+    if (mSelStart != -1) {
+        SelHighlight(mSelStart, mSelEnd);
+        mSelStart = -1;
+    }
+}
+
+// User settable table: what characters are to be considered alphabetic?
+// 256 bits
+__u32 Console::mInWordLut[8] = {
+  0x00000000, // control chars
+  0x03FF0000, // digits
+  0x87FFFFFE, // uppercase and '_'
+  0x07FFFFFE, // lowercase
+  0x00000000,
+  0x00000000,
+  0xFF7FFFFF, // latin-1 accented letters, not multiplication sign
+  0xFF7FFFFF  // latin-1 accented letters, not division sign
+};
+
+int Console::InWord(const unsigned char c) {
+    return ( mInWordLut[c>>5] >> (c & 0x1F) ) & 1;
+}
+
+// does screen address p correspond to character at LH/RH edge of screen?
+int Console::AtColEdge(const int p)
+{
+    return (!(p % mMaxCols) || !((p + 1) % mMaxCols));
+}
+
+// Don't take this from <ctype.h>: 011-015 on the screen aren't spaces
+#define IS_SPACE(c)  ((c) == ' ')
+#define IS_ASCII(c)  ((c) & txtASCII)
+
+// based on kernel, linux/drivers/char/selection.c
+void Console::SelCopy(int c1, int r1, int c2, int r2, int mode) {
+    int xs, ys, xe, ye, ps, pe;
+    xs = CorrectCol(c1);
+    ys = CorrectRow(r1);
+    xe = CorrectCol(c2);
+    ye = CorrectRow(r2);
+    ps = ys * mMaxCols + xs;
+    pe = ye * mMaxCols + xe;
+    //debug<<"Mouse " <<xs<<","<<ys<<" "<<xe<<","<<ye<<" "
+    //    <<ps<<","<<pe<<" "<<endl;
+
+    if (mode == 4) {  // useful for screendump without selection highlights
+        SelClear();
+        return;
+    }
+
+    if (ps > pe)    // make exchange if sel_start > sel_end
+    {
+        int tmp = ps;
+        ps = pe;
+        pe = tmp;
+    }
+
+    int new_sel_start, new_sel_end;
+    bool spc, ascii;
+    char *bp, *obp;
+    int i;
+
+    switch (mode)
+    {
+        case 0: // character-by-character selection
+            new_sel_start = ps;
+            new_sel_end = pe;
+            break;
+        case 1: // word-by-word selection
+            spc = IS_SPACE(mpText[ps]);
+            ascii = IS_ASCII(mpFlag[ps]);
+            for (new_sel_start = ps; ; ps--)
+            {
+                if (ascii) {
+                    if (spc && !IS_SPACE(mpText[ps]))
+                        break;
+                    if (!spc && !InWord(mpText[ps]))
+                        break;
+                } else {
+                    if (IS_ASCII(mpFlag[ps]))
+                        break;
+                }
+                new_sel_start = ps;
+                if (!(ps % mMaxCols))
+                    break;
+            }
+            spc = IS_SPACE(mpText[pe]);
+            ascii = IS_ASCII(mpFlag[pe]);
+            for (new_sel_end = pe; ; pe++)
+            {
+                if (ascii) {
+                    if (spc && !IS_SPACE(mpText[pe]))
+                        break;
+                    if (!spc && !InWord(mpText[pe]))
+                        break;
+                } else {
+                    if (IS_ASCII(mpFlag[pe]))
+                        break;
+                }
+                new_sel_end = pe;
+                if (!((pe+1) % mMaxCols))
+                    break;
+            }
+            break;
+        case 2: // line-by-line selection
+            new_sel_start = ps - ps % mMaxCols;
+            new_sel_end = pe + mMaxCols - pe % mMaxCols - 1;
+            break;
+        case 3:
+            SelPointer(pe);
+            return;
+        default:
+            return;
+    }
+
+    // remove the pointer
+    SelPointer(-1);
+
+    // select to end of line if on trailing space
+    if (new_sel_end > new_sel_start &&
+        !AtColEdge(new_sel_end) &&
+        IS_SPACE(mpText[new_sel_end])) {
+        for (pe = new_sel_end + 1; ; pe++)
+            if (!IS_SPACE(mpText[pe]) ||
+                AtColEdge(pe))
+                break;
+        if (IS_SPACE(mpText[pe]))
+            new_sel_end = pe;
+    }
+    if (mSelStart == -1)    // no current selection
+        SelHighlight(new_sel_start, new_sel_end);
+    else if (new_sel_start == mSelStart)
+    {
+        if (new_sel_end == mSelEnd) // no action required
+            return;
+        else if (new_sel_end > mSelEnd) // extend to right
+            SelHighlight(mSelEnd + 1, new_sel_end);
+        else                // contract from right
+            SelHighlight(new_sel_end + 1, mSelEnd);
+    }
+    else if (new_sel_end == mSelEnd)
+    {
+        if (new_sel_start < mSelStart)  // extend to left
+            SelHighlight(new_sel_start, mSelStart - 1);
+        else                // contract from left
+            SelHighlight(mSelStart, new_sel_start - 1);
+    }
+    else    // some other case; start selection from scratch
+    {
+        SelClear();
+        SelHighlight(new_sel_start, new_sel_end);
+    }
+    mSelStart = new_sel_start;
+    mSelEnd = new_sel_end;
+
+    // Allocate a new buffer before freeing the old one ...
+    bp = new char[mSelEnd-mSelStart+1];
+    if (!bp) { // selection: kmalloc() failed
+        SelClear();
+        return;
+    }
+    if (mpSelBuf)
+        delete[] mpSelBuf;
+    mpSelBuf = bp;
+
+    obp = bp;
+    for (i = mSelStart; i <= mSelEnd; i++) {
+        *bp = mpText[i];
+        if (!IS_SPACE(*bp++))
+            obp = bp;
+        if (! ((i + 1) % mMaxCols)) {
+            // strip trailing blanks from line and add newline,
+            //   unless non-space at end of line.
+            if (obp != bp) {
+                bp = obp;
+                *bp++ = '\r';
+            }
+            obp = bp;
+        }
+    }
+    mSelBufLen = bp - mpSelBuf;
+}
+
+void Console::SelPaste(int fd) {
+    assert(fd >= 0);
+    int pasted = 0, count;
+    while (mpSelBuf && (mSelBufLen > pasted)) {
+        count = mSelBufLen - pasted;
+        pasted = write(fd, mpSelBuf, count);
+        pasted += count;
+    }
+}
+
