@@ -20,7 +20,7 @@
 
 #ifndef NDEBUG
 #include <fstream>
-extern ofstream debug;
+extern std::ofstream debug;
 #endif
 
 #endif
