@@ -58,7 +58,11 @@ inline static int rgb8to16 (int rgb)
     return rgb * (0xFFFF / 0xFF);
 }
 
-bool LIBGGI::TryOpen(char *ggiDriveName) {
+void LIBGGI::Update() {
+    ggiFlush(visual);
+}
+
+bool LIBGGI::TryOpen() {
     int i;
     const ggi_pixelformat* pf;
 
@@ -67,14 +71,19 @@ bool LIBGGI::TryOpen(char *ggiDriveName) {
         return false;
     }
 
-    visual = ggiOpen (ggiDriveName);
+    visual = ggiOpen (NULL);
     if (visual == NULL) {
         printf("unable to open default visual, exiting.\n");
         ggiExit();
         return false;
     }
 
-    // ggiSetFlags (visual, GGIFLAG_ASYNC);
+     /* Using ASYNC mode can give great performance improvements on some
+      * targets so it should be used as much as possible.  Turning ASYNC mode on right
+      * after ggiOpen() ensures best operation
+      */
+
+    ggiSetFlags (visual, GGIFLAG_ASYNC);
     ggiCheckMode (visual, &mode);
     if (ggiSetMode (visual, &mode))    { 
         printf("Can't set mode\n");
