@@ -26,7 +26,7 @@
 
 class LIBGGI : public GraphDev  {
     public:
-        static bool TryOpen(char *ggiDriveName);
+        static bool TryOpen();
         virtual ~LIBGGI();
 
         void PutPixel(int x,int y,int color);
@@ -36,6 +36,9 @@ class LIBGGI : public GraphDev  {
 
         void SwitchToGraph();
         void SwitchToText();
+        void Update();
+        string Name() { return "ggi"; }
+
     private:
         int GetPixMap(char* expanded,int fg,int bg,struct CharBitMap* pFont);
 
