@@ -25,6 +25,7 @@
 #include <sys/ioctl.h>
 #include <sys/mman.h>
 #include <assert.h>
+#include <iostream>
 #include "debug.h"
 
 #include "display/fbdev.h"
@@ -35,6 +36,7 @@
     #endif
 #endif
 
+using namespace std;
 // mmap framebuffer address
 GraphDev *GraphDev::mpGraphDev = NULL;
 
@@ -50,46 +52,59 @@ int GraphDev::mBlankLineHeight = 0;
 struct CharBitMap GraphDev::mAsc = {0};
 struct CharBitMap GraphDev::mDbl = {0};
 
-bool GraphDev::Open() {
-#ifdef HAVE_GGI_LIB
-    #ifndef NDEBUG
-    //try libggi first for debug
-    char *ggiDriveName = getenv("GGI_DISPLAY");
-    if (ggiDriveName) {
-        if (LIBGGI::TryOpen(ggiDriveName))
+bool GraphDev::Open(char* drv) {
+    if (drv == string("auto")) {
+        /* auto:fbdev->ggi->vga */
+        cerr <<  "trying fbdev driver" << endl;
+        OPEN_RC rc = FBDev::TryOpen();
+        if (rc == NORMAL)
             return true;
-        throw(runtime_error("ggi open error, check env GGI_DISPLAY!"));
-    }
-    #endif
-#endif
-    //first try fbdev
-    OPEN_RC rc = FBDev::TryOpen();
-    switch (rc) {
-        case NORMAL:
-            return true;
-        case UNSUPPORT:
+        else 
+            cerr << "Can not open FrameBuffer device on this machine." << endl;
+
+        /* now ggi */
 #ifdef HAVE_GGI_LIB
-            //try libggi
-            {
-            char *ggiDriveName = getenv("GGI_DISPLAY");
-            if (ggiDriveName)
-                if (LIBGGI::TryOpen(ggiDriveName))
-                    return true;
-            }
+        cerr << "trying ggi driver" << endl;
+        if (LIBGGI::TryOpen())
+            return true;
+        else
+            cerr << "Can not open libggi on this machine." << endl;
 #endif
-            //libggi failed,now try vga
-        case FAILURE:
+
+        /* finally vga */
 #if defined(linux)
 #ifdef USING_VGA
-            if (VGADev::TryOpen())
-                return true;
-            break;
+        if (VGADev::TryOpen())
+            return true;
 #endif
 #endif
-        default:  // vga fail
+        return false;
+    } else if (drv == string("ggi")) {
+#ifdef HAVE_GGI_LIB
+        if (LIBGGI::TryOpen())
+            return true;
+        else 
+            cerr << "Can not open libggi on this machine." << endl;
+#endif
             return false;
+    } else if (drv == string("fb")) {
+        OPEN_RC rc = FBDev::TryOpen();
+        if (rc == NORMAL)
+            return true;
+        else 
+            cerr << "Can not open FrameBuffer device on this machine." << endl;
+        return false;
+    } else if (drv == string("vga")) {
+#if defined(linux)
+#ifdef USING_VGA
+        if (VGADev::TryOpen())
+            return true;
+#endif
+#endif
+        return false;
+    } else {
+        throw runtime_error("unknown video driver");
     }
-    return false;
 }
 
 bool GraphDev::Open(int xres, int yres, int depth) {
