@@ -42,7 +42,7 @@ enum OPEN_RC { NORMAL, UNSUPPORT, FAILURE };
 
 class GraphDev {
     public:
-        static bool Open();
+        static bool Open(char* drv = "auto");
         static bool Open(int xres, int yres, int depth);
         static void Close();
         virtual ~GraphDev() {};
@@ -82,6 +82,9 @@ class GraphDev {
 
         virtual void SwitchToGraph() {};
         virtual void SwitchToText() {};
+        virtual void Update() {};
+        virtual string Name() = 0;
+
     protected:
         static int mXres, mYres;
         // font
