@@ -37,7 +37,7 @@ void InputClient::SetColor(string & s) {
     Pos = 0;
     while ( Pos < s.length() ) {
         NextPos = s.find(',', Pos);
-        if (NextPos == string::npos)
+        if (NextPos == (unsigned int)string::npos)
             NextPos = s.length();
         sVal = s.substr(Pos,NextPos - Pos);
         // debug << atoi(sVal.c_str()) << endl;
