@@ -37,10 +37,11 @@ using namespace std;
 #ifdef HAVE_UNICON_LIB
     #include "uniconinputserver.h"
 #endif
-#include "configserver.h"
 #include "nativebarclient.h"
 #include "overspotclient.h"
 #include "inputmanager.h"
+#include "encfilter.h"
+#include "iconv_string.h"
 
 int InputManager::mConFd = -1;
 int InputManager::mConNo = -1;
@@ -54,7 +55,10 @@ void InputManager::SetTty(int confd, int ttyno, int ttyfd) {
 
 InputManager::InputManager( Console * pCon,
     InputStyle style, string& OverSpotColors, string& NativeBarColors)
-        : mpCon(pCon),
+        :
+        mInputRead(0), 
+        mOutputRead(0), 
+        mpCon(pCon),
         mpInputServer(NULL),
         mpInputClient(NULL),
         mActive(false),
@@ -94,10 +98,8 @@ void InputManager::Process(InputEvt &evt) {
     evt.oper = InputEvt::Nothing;
     evt.sub = 0;
     
-    const int BUFSIZE = 8192;
-    char buf[BUFSIZE];
     struct timeval tv;
-    int count, rcved, shift;
+    int rcved, shift;
 
     shift = GetShiftState();
     if (shift != 0) {
@@ -128,18 +130,47 @@ void InputManager::Process(InputEvt &evt) {
     }
 #endif
 
-   if (FD_ISSET(mConFd, &mFdSet)) {
-        count = read(mConFd, buf, BUFSIZE);
-        if (count > 0) {
-            for (int i = 0; i < count; i++)
-               ProcessKey(buf[i], evt);
+    if (FD_ISSET(mConFd, &mFdSet)) {
+        mOutputRead += read(mConFd, mOutputBuf+mOutputRead, BUFSIZE-mOutputRead);
+        if (mOutputRead > 0) {
+#ifdef HAVE_ICONV
+            /* XXX: under utf8 mode the system hotkey (single char) will be converted into
+             * some crazy 3-byte sequence. To avoid this problem, an ad hoc workaround is to only
+             * perform encoding conversion on buffer with more than 1 byte.
+             */
+            if (mOutputRead > 1 && UseEncodingFilter) {
+                mOutputRead = DoEncodingFilter(CONVERT_TO_UTF8_FILTER, mOutputBuf, mOutputRead);
+                if (EncodingFilterLen > 0) {
+                    for (size_t i = 0; i < EncodingFilterLen; i++)
+                        ProcessKey(EncodingFilterBuff[i], evt);
+                }
+            }
+            else
+#endif
+            {
+                for (int i = 0; i < mOutputRead; i++)
+                    ProcessKey(mOutputBuf[i], evt);
+                mOutputRead = 0;
+            }
         }
-   }
+    }
 
    if (FD_ISSET(mTtyFd, &mFdSet)) {
-        count = read(mTtyFd, buf, BUFSIZE);
-        if (count > 0) {
-            mpCon->Write(buf, count);
+        mInputRead += read(mTtyFd, mInputBuf+mInputRead, BUFSIZE-mInputRead);
+        if (mInputRead > 0) {
+#ifdef HAVE_ICONV
+            if (UseEncodingFilter) { /* all the utf8 data go through filter to system encoding */
+                mInputRead = DoEncodingFilter(CONVERT_FROM_UTF8_FILTER, mInputBuf, mInputRead);
+                if (EncodingFilterLen > 0) {
+                    mpCon->Write(EncodingFilterBuff, EncodingFilterLen);
+                }
+            }
+            else
+#endif
+            {
+                mpCon->Write(mInputBuf, mInputRead);
+                mInputRead = 0;
+            }
         }
     }
 }
@@ -185,8 +216,21 @@ void InputManager::ProcessKey(char c, InputEvt &evt) {
             DoCtrlSpace();
             return;
         }
-        if (!s.empty())
+        if (!s.empty()) {
+            if (UseEncodingFilter) {
+                /* need to convert output from input method into UTF-8 */
+                /* TODO: need a better way to detech UseEncodingFilter */
+                char* result = NULL;
+                size_t len;
+                int rc;
+                rc = iconv_string ("UTF-8", "GBK", s.c_str(), s.c_str()+s.length()+1, &result, &len);
+                if (rc == 0)
+                    s = result;
+                if (result)
+                    free(result);
+            }
             WriteClient(s);
+        }
         mpInputClient->Update();
         return ;
     }
@@ -236,9 +280,6 @@ bool InputManager::ProcessSysKey(char c, InputEvt &evt) {
             evt.oper = InputEvt::AutoEncoSwitch;
             evt.sub = 0;
             break;
-        case CTRL_F10:
-            MenuMode();
-            break;
         case SHIFT_PAGEUP:
             mpCon->ScrollDelta(Console::PAGE_UP);
             break;
@@ -289,6 +330,7 @@ bool InputManager::ProcessInputKey(char c) {
             //case CTRL_F7:
             //break;
         case CTRL_SPACE:
+        case 0: // CTRL_SPACE under GGI-X
             DoCtrlSpace();
             break;
         case CTRL_SHIFT:
@@ -397,6 +439,7 @@ bool InputManager::LoadImm(ImmInfo& rInfo) {
             mpCurImm = NULL; //force reload
             bool r = LoadImm(*pOld);
             assert(r);
+            r = true; /* remove warning of unused r */
         }
         return false;
     }
@@ -416,7 +459,7 @@ void InputManager::LoadImmInfo(ConfigFile& f) {
         Pos = 0;
         while ( Pos < s.length() ) {
             NextPos = s.find(',', Pos);
-            if (NextPos == string::npos)
+            if (NextPos == (unsigned int)string::npos)
                 NextPos = s.length();
             sVal = s.substr(Pos, NextPos - Pos);
             switch (ParmNo) {
@@ -477,6 +520,7 @@ void InputManager::DoCtrlSpace() {
     }
 }
 
+#if 0
 void InputManager::MenuMode() {
     if (mpInputServer)
         delete mpInputServer;
@@ -486,6 +530,7 @@ void InputManager::MenuMode() {
     mpInputClient->Show();
     mpInputClient->Update();
 }
+#endif
 
 void InputManager::Show() {
     if (mActive && mClientVisible) {
@@ -788,7 +833,7 @@ void InputManager::HelpShow() {
     mpHelpWin->PutStr(45, 6,  _("F4  JIS"));
     mpHelpWin->PutStr(1, 7,   _("F5  KSC"));
     mpHelpWin->PutStr(15, 7,  _("F9  Switch & Auto Detect"));
-    mpHelpWin->PutStr(45, 7,  _("F10 Sys Menu"));
+    // mpHelpWin->PutStr(45, 7,  _("F10 Sys Menu"));
     mpHelpWin->PutStr(1, 8,   _("F7  Input Style"));
     mpHelpWin->PutStr(45, 8,  _("D   Exit zhcon"));
 
