@@ -29,11 +29,11 @@ ImmInfo InputServer::GetImmInfo() {
     return mImmInfo;
 }
 
-inline bool InputServer::IsFullChar() {
+bool InputServer::IsFullChar() {
     return mIsFullChar;
 }
 
-inline bool InputServer::IsFullComma() {
+bool InputServer::IsFullComma() {
      return mIsFullComma;
 }
 void InputServer::SetClientBufLen(int len){
