@@ -43,7 +43,7 @@ GraphDev *gpScreen = NULL;
 HzDecoder *gpDecoder = NULL;
 Zhcon *gpZhcon = NULL;
 
-int main(int argc, char *argv[]) {
+int main(int argc, char* argv[]) {
     try {
         Zhcon con(argc, argv);
         gpZhcon = &con;
@@ -51,7 +51,10 @@ int main(int argc, char *argv[]) {
         con.Run();
     } catch (runtime_error & e) {
         cerr << e.what() << endl;
+    } catch (...) {
+        cerr << "unknown exception caught" << endl;
     }
+
     return EXIT_SUCCESS;
 }
 
