@@ -86,10 +86,11 @@ void NativeBarClient::Update() {
     } else {
         mpServer->GetCandilist(mList);
 
-        mTitle = _("[");
-        mTitle += (mpServer->IsFullChar() ? _("Full") : _("Half"));
-        mTitle += (mpServer->IsFullComma() ? _("Ch") : _("En"));
-        mTitle += _("]") + (mpServer->GetImmInfo()).mName;
+//        mTitle = _("[");
+//        mTitle += (mpServer->IsFullChar() ? _("Full") : _("Half"));
+//        mTitle += (mpServer->IsFullComma() ? _("Ch") : _("En"));
+//        mTitle += _("]") +
+        mTitle = (mpServer->GetImmInfo()).mName;
         mTitle += " ";
 
         char buf[64];
@@ -180,16 +181,12 @@ void NativeBarClient::Connect(InputServer *pServer) {
 }
 
 string NativeBarClient::GetPromptStr() {
-    string tip = gpZhcon->GetATip();
     string encode = gpZhcon->GetEncode();
     string s;
     s = "  ";
     s += PACKAGE;
     s += " ";
     s += VERSION;
-    s += "  [";
-    s += tip;
-    s += "]";
     s.resize(mMaxCols - encode.size() - 4, ' ');
     s += _("[");
     s += encode;
