@@ -64,10 +64,11 @@ void OverSpotClient::Update() {
         mpServer->GetCandilist(mList);
 
         if (mList.mCount == 0) {
-            mTitle = _("[");
-            mTitle += (mpServer->IsFullChar() ? _("Full") : _("Half"));
-            mTitle += (mpServer->IsFullComma() ? _("Ch") : _("En"));
-            mTitle += _("]") + (mpServer->GetImmInfo()).mName;
+//            mTitle = _("[");
+//            mTitle += (mpServer->IsFullChar() ? _("Full") : _("Half"));
+//            mTitle += (mpServer->IsFullComma() ? _("Ch") : _("En"));
+//            mTitle += _("]") + (mpServer->GetImmInfo()).mName;
+            mTitle = _("[") + (mpServer->GetImmInfo()).mName + _("]");
         } else {
             mTitle = _("[") + (mpServer->GetImmInfo()).mName + _("]");
         }
