@@ -54,6 +54,7 @@ mFgColor(7), mBgColor(0) {
         memset(mpOverlaps, 0, size);
     }
 
+    SetCursorType(CUR_DEF);
     mVisible = false;
 }
 
@@ -97,6 +98,7 @@ Window::~Window() {
 }
 
 void Window::SetCursorType(int CurType) {
+    this->CursorRestore();
     switch (CurType) {
         case CUR_NONE:
             mCursorStart = gpScreen->BlockHeight() - 1;
@@ -127,14 +129,14 @@ void Window::SetCursorType(int CurType) {
     }
 }
 
-inline int Window::ColsOvered() {
+int Window::ColsOvered() {
     int cols = Width() / gpScreen->BlockWidth();
     if (Width() % gpScreen->BlockWidth())
         cols++;
     return cols;
 }
 
-inline int Window::RowsOvered() {
+int Window::RowsOvered() {
     int rows = Height() / gpScreen->BlockHeight();
     if (Height() % gpScreen->BlockHeight())
         rows++;
@@ -207,7 +209,7 @@ void Window::PutChar(char attr,char flag,char c1,char c2){
 }
 
 //return index of (r,c) in mpText,mpAttr,mpFlag
-inline int Window::Index(int c, int r) {
+int Window::Index(int c, int r) {
     return c + r * mMaxCols;
 }
 
@@ -297,6 +299,32 @@ void Window::RedrawRow(int row, char *pText, char *pAttr, char *pFlag, bool isUp
     }
 }
 
+void Window::RedrawChar(int col, int row) {
+    assert(col >= 0 && col <= mEndCol && row >= 0 && row <= mEndRow);
+
+    int idx = Index(col, row);
+
+    mpFlag[idx] &= ~txtUpdated;
+    if ((mpFlag[idx] & txtDblCode1) && col < mpConsole->MaxCols() - 1
+        && (mpFlag[idx + 1] & txtDblCode2) ) {
+        //full hz out
+        OutChar(col, row, FgColor(mpAttr[idx]), BgColor(mpAttr[idx]),
+                mpText[idx], mpText[idx + 1]);
+        return;
+    }
+    if ((mpFlag[idx] & txtDblCode2) && col > 0
+        && (mpFlag[idx - 1] & txtDblCode1) ) {
+        //full hz out
+        OutChar(col-1, row, FgColor(mpAttr[idx]), BgColor(mpAttr[idx]),
+                mpText[idx-1], mpText[idx]);
+        return;
+    }
+    
+    //ascii or single double code
+    OutChar(col, row, FgColor(mpAttr[idx]), BgColor(mpAttr[idx]),
+            mpText[idx]);
+}
+
 //implement blink attribute on console
 void Window::UpdateBlinkAttr(bool show) {
     int size = mpConsole->MaxCols() * mpConsole->MaxRows();
@@ -596,6 +624,7 @@ void Window::CursorSet() {
         return;
     // more rapid when fresh screen
     // CursorBlink();
+    //debug<<"(CursorSet)"<<flush;
     if (mCursorVisible == false) {
         CursorDraw();
         mCursorVisible = true;
@@ -605,6 +634,7 @@ void Window::CursorSet() {
 void Window::CursorRestore() {
     if (mCursorEnabled == false)
         return;
+    //debug<<"(CursorRestore)"<<flush;
     if (mCursorVisible == true) {
         CursorDraw();
         mCursorVisible = false;
