@@ -20,7 +20,6 @@
 #endif
 
 #include <iostream>
-#include <strstream>
 #include <unistd.h>
 #include <cstdlib>
 #include <cstdio>
@@ -51,6 +50,7 @@
 #include "global.h"
 #include "console.h"
 
+#include "encfilter.h"
 #include "inputclient.h"
 #include "inputmanager.h"
 #include "nativeinputserver.h"
@@ -72,6 +72,8 @@
 //#include "fade.h"
 #include "zhcon.h"
 #include "basefont.h"
+#include "encfilter.h"
+#include "cmdline.h"
 //#include "popwin.h"
 BaseFont* gpHzFont;
 BaseFont* gpAscFont;
@@ -85,21 +87,6 @@ Zhcon* Zhcon::mpZhcon = NULL;
 int Zhcon::mTtyPid = 0;
 Zhcon::STATE Zhcon::mState = STOP;
 
-char* Zhcon::mpTips[] = {
-gettext_noop("Please visit zhcon.gnuchina.org for more information."),
-gettext_noop("Zhcon supports GB2312,GBK,BIG5,JIS,KSC encode."),
-gettext_noop("You can use CTRL-ALT-0 to toggle prompt bar."),
-gettext_noop("Press CTRL-ALT-H for online help."),
-gettext_noop("Zhcon supports 12,14,16,24 pixel fonts,you can change in zhcon.conf"),
-gettext_noop("You can use 24 pixel font on 17 monitor to get better visual effect."),
-gettext_noop("Hit CTRL-D to quit zhcon."),
-gettext_noop("You can find user manual in doc/."),
-gettext_noop("Zhcon can use input methods from Unicon[TurboLinux],read manual for detail."),
-gettext_noop("Zhcon is optimized for running under Linux FrameBuffer."),
-gettext_noop("Start from version 0.2.1,zhcon support gpm mouse in console."),
-gettext_noop("Press CTRL-F7 to change input style")
-};
-
 void Zhcon::SignalVtLeave(int signo) {
     if(Zhcon::mpZhcon != NULL) mpZhcon->VtDoLeave();
 }
@@ -110,16 +97,22 @@ void Zhcon::SignalVtEnter(int signo) {
 
 Zhcon::Zhcon(int argc, char* argv[])
 :mpCon(NULL),
-mpInputManager(NULL),
-mShowTips(true),
-mArgc(argc),
-mArgv(argv)
+mpInputManager(NULL)
 {
     Zhcon::mpZhcon = this;
     mConFd = mTtyFd = -1;
-    random_shuffle(mpTips,mpTips + sizeof(mpTips)/sizeof(char*) );
+
+    /* let's call our CMDLINE Parser */
+    if (cmdline_parser(argc, argv, &mArgs) != 0) {
+        cmdline_parser_print_help();
+        throw runtime_error("fail to parse command line arguments");
+    }
+
+    if (mArgs.utf8_flag)
+            UseEncodingFilter = 1;
 }
 
+// will be called on exception
 Zhcon::~Zhcon() {
     CleanUp();
     Zhcon::mpZhcon = NULL;
@@ -130,7 +123,7 @@ void Zhcon::Init() {
     string cfgfile = getenv("HOME");
     cfgfile += "/.zhconrc";
     if (access(cfgfile.c_str(), R_OK) != 0)
-        cfgfile = "/etc/zhcon.conf";
+        cfgfile = PREFIX "/etc/zhcon.conf";
 
     //for debug,a pause enable us to attach zhcon's pid in gdb
     //char c;cin>>c;
@@ -141,6 +134,7 @@ void Zhcon::Init() {
     // set blank line height, must before init font
     InitGraphDev(f);
     GraphMode();
+	seteuid(getuid());
     InitLocale(f); // include init font
     InitCon(f);
     InitMisc(f);
@@ -165,6 +159,7 @@ void Zhcon::Run() {
     while (mState == RUNNING) {
         mpCon->CursorBlink();
         mpInputManager->Process(evt);
+        gpScreen->Update(); //   force ggi to update screen
         if (evt.oper != InputEvt::Nothing) {
             DoInputEvt(evt);
             continue;
@@ -242,7 +237,6 @@ void Zhcon::DoInputEvt(InputEvt &evt) {
 }
 
 //fork a new pty to run user's shell
-//return true if successfully fork
 void Zhcon::ForkPty() {
     char name[50];
     mTtyPid = forkpty(&mTtyFd, name, NULL, NULL);
@@ -258,20 +252,29 @@ void Zhcon::ForkPty() {
             throw runtime_error("can not get user's shell!");
 
         /* close all opened file */
-#ifndef OPEN_MAX
-    #warning OPEN_MAX undefined so far,try define it to 64
-    #define OPEN_MAX 64
-#endif
-        for (int i = 3; i < OPEN_MAX; i++)
+        int open_max = sysconf(_SC_OPEN_MAX);
+        for (int i = 3; i < open_max; i++)
             close(i);
         setsid();
-        seteuid(getuid());   /* for security */
+        // seteuid(getuid());   /* for security */
+
         //TODO: handle error condition
-        if (mArgc > 1)
-            execvp(mArgv[1], &mArgv[1]);
-        else
+        if (mArgs.inputs_num > 0) {
+            // the second arg of execvp must be terminated by a NULL pointer
+            // we have to create a new one with NULL pointer at the end
+            char** args = (char**)malloc(sizeof(char*)*(mArgs.inputs_num+1));
+            for (size_t i = 0; i < mArgs.inputs_num; ++i)
+                args[i] = mArgs.inputs[i];
+            args[mArgs.inputs_num] = NULL;
+            cout << "Executing [";
+            for (size_t i = 0; i < mArgs.inputs_num; ++i)
+                cout << ' ' << args[i] << ' ';
+            cout << ']' << endl;
+            execvp(args[0], args);
+        } else
             execlp(userpd->pw_shell, userpd->pw_shell, (char *) 0);
-        exit(-1);
+
+        exit(0);
     }
 }
 
@@ -301,7 +304,8 @@ void Zhcon::InstallSignal() {
     //    sigaction(SIGALRM, &act, NULL);
 }
 
-//do some clean up before quit
+// as CleanUp() is also called on exception, care should be made so that
+// only full inititlized part are cleaned
 void Zhcon::CleanUp() {
     struct sigaction act;
     /* done in procress and block all serious signal to prevent interrupt */
@@ -351,6 +355,9 @@ void Zhcon::CleanUp() {
         close(mConFd);
     }
 
+    if (mArgs.utf8_flag)
+        CleanupEncodingFilter();
+
     setenv("LC_ALL", mOldLocale.c_str(), 1);
 }
 
@@ -373,8 +380,12 @@ void Zhcon::InitTty() {
     
     if (!(strncmp("/dev/tty", TtyName, 8) == 0 ||
         strncmp("/dev/vc/", TtyName, 8) == 0)) {
-        printf("%s is not real tty or vc. Please exit current tty and try again.\r\n", TtyName);
+        fprintf(stderr, "warning!!!\n");
+        fprintf(stderr, "%s is not real tty or vc, are your running under X-Window?\n", TtyName);
+#ifndef HAVE_GGI_LIB
+        fprintf(stderr, "libggi support not complied in, can not run under X-Window, now quit\n");
         exit(1);
+#endif
     }
     
     mConFd = open(TtyName, O_RDWR);
@@ -529,10 +540,14 @@ void Zhcon::SetEncode(Encode e, Encode font) {
 
 void Zhcon::VtSignalSet(int mode)
 {
-    //return; for debug
+    // Signal handing only apply to console mode (fb or vga), after GraphDev being correctly initialized, 
+    // not applied to ggi
+    if (GraphDev::mpGraphDev == NULL || GraphDev::mpGraphDev->Name() == "ggi")
+        return;
+
     vt_mode vtm;
     if (ioctl(mConFd, VT_GETMODE, &vtm))
-        throw runtime_error("ioctl VT_GETMODE failed!");
+        throw runtime_error("in Zhcon::VtSignalSet() ioctl VT_GETMODE failed!");
 
     switch (mode) {
         case 1:  // leave
@@ -549,6 +564,7 @@ void Zhcon::VtSignalSet(int mode)
             vtm.acqsig = SIGUSR1;
             vtm.frsig = SIGUSR1;
             break;
+        case 0:
         default:  // clean
             signal(SIGUSR1, SIG_DFL);
             vtm.mode = VT_AUTO;
@@ -556,7 +572,7 @@ void Zhcon::VtSignalSet(int mode)
     }
     
     if (ioctl(mConFd, VT_SETMODE, &vtm))
-        throw runtime_error("ioctl VT_SETMODE failed!");
+        throw runtime_error("in Zhcon::VtSignalSet() ioctl VT_SETMODE failed!");
 }
 
 void Zhcon::VtDoLeave() {
@@ -626,16 +642,23 @@ void Zhcon::InitLocale(ConfigFile& f){
 #if defined (__FreeBSD__)
         setenv("LC_ALL", "zh_CN.EUC", 1);
 #else
-        setenv("LC_ALL", "zh_CN.GB2312", 1);
+        if (mArgs.utf8_flag)
+            setenv("LC_ALL", "zh_CN.UTF-8", 1);
+        else
+            setenv("LC_ALL", "zh_CN.GB2312", 1);
 #endif
         mDefaultEncode = GB2312;
     } else if (s == "gbk") {
-        //SetEncode(GBK,GBK);
-        setenv("LC_ALL", "zh_CN.GBK", 1);
+        if (mArgs.utf8_flag)
+            setenv("LC_ALL", "zh_CN.UTF-8", 1);
+        else
+            setenv("LC_ALL", "zh_CN.GBK", 1);
         mDefaultEncode = GBK;
     } else if (s == "big5") {
-        //SetEncode(BIG5,BIG5);
-        setenv("LC_ALL", "zh_TW.Big5", 1);
+        if (mArgs.utf8_flag)
+            setenv("LC_ALL", "zh_TW.UTF-8", 1);
+        else
+            setenv("LC_ALL", "zh_TW.Big5", 1);
         mDefaultEncode = BIG5;
     } else if (s == "jis") {
         //SetEncode(JIS,JIS);
@@ -649,6 +672,10 @@ void Zhcon::InitLocale(ConfigFile& f){
         throw runtime_error("unable to set default encode!");
     }
 
+    if (mArgs.utf8_flag) {
+        SetupEncodingFilter(s.c_str());
+    }
+
     s = f.GetOption(string("autoencode"), string("manual"));
     if (s == "auto")
         mAutoEncode = AUTO;
@@ -663,7 +690,7 @@ void Zhcon::InitLocale(ConfigFile& f){
 void Zhcon::InitGraphDev(ConfigFile& f){
     bool r;
 #if defined(linux)
-    r = GraphDev::Open();
+    r = GraphDev::Open(mArgs.drv_arg);
 #elif defined(__FreeBSD__)
     int xres = f.GetOption(string("x_resolution"), 640);
     int yres = f.GetOption(string("y_resolution"), 480);
@@ -671,9 +698,20 @@ void Zhcon::InitGraphDev(ConfigFile& f){
     r = GraphDev::Open(xres, yres, depth);
 #endif
     if (!r)
-        throw(runtime_error("Can not open graph device,make sure "
-            "you have a kernel compiled with framebuffer support or "
-            "you have a vga card."));
+        throw(runtime_error(
+            "\n============== I'm really sorry, but... ================\n"
+            "I can not open graphical device on this machine, this can happen when:\n"
+            "1. your kernel does not have framebuffer device enabled, check the output from `dmesg|grep vesa`\n"
+            "2. you are running on a non-i386 machine so no VGA support\n"
+            "3. you are running under X-Window but libggi is not compiled in (required for running zhcon under X-Window)\n"
+            "\n"
+            "Don't be panic by this message, thousands of people have run zhcon successfully, surely you can!\n"
+            "I suggest you visit http://zhcon.sourceforge.net for more information, or send an email to zhcon-users@lists.sourceforge.net\n"
+            "I'm pretty sure your problem will be solved very quickly\n"
+            "You can subscribe to the list on https:// lists.sourceforge.net/lists/listinfo/zhcon-users\n"
+            "\n"
+            "Good Luck!\n"
+            ));
     gpScreen = GraphDev::mpGraphDev;
     GraphDev::mBlankLineHeight = f.GetOption(string("blanklineheight"), 0);
     if (GraphDev::mBlankLineHeight < 0)
@@ -748,23 +786,12 @@ void Zhcon::StartupMsg(){
     s = VERSION;
     s.resize(7,' ');
     scr.replace(scr.find("VERSION"),7,s);
-    strstream buf;
-    buf<<_("screen resolution")<<": "<<gpScreen->Width()<<"X"<<
-        gpScreen->Height()<<" ["<<mpCon->MaxCols()<<","<<
-        mpCon->MaxRows()<<"]"<<"\r\n";
-    buf<<_("default encode")<<": "<<GetEncode()<<"\r\n";
-    /*
-    if (mpInputManager->mMouse.mFd >= 0) {
-        buf<<_("Mouse")<<": "<<_("On");
-        if (mpInputManager->mMouse.GetButtons() == 3)
-            buf<<", 3 " << _("buttons");
-        buf<<"\r\n";
-    }
-    else
-        buf<<_("Mouse")<<": "<<_("Off")<<"\r\n";
-    */
-    // no pcount, may display garbage
-    scr += string(buf.str(), buf.pcount());
+    char buf[1024];
+
+    sprintf(buf, "screen resolution: %dx%d [col=%d, row=%d]\r\ndefault encoding: %s %s\r\n", 
+            gpScreen->Width(), gpScreen->Height(), mpCon->MaxCols(), mpCon->MaxRows(), 
+            GetEncode().c_str(), (mArgs.utf8_flag?" [UTF-8]":""));
+    scr += buf;
     mpCon->Write(scr.c_str(),scr.size());
 }
 
@@ -808,11 +835,3 @@ string Zhcon::GetEncode(){
     }
     return s;
 }
-//return a random tip string
-string Zhcon::GetATip(){
-    static unsigned i = 0;
-    if (!mShowTips) return "";
-    if (++i >= (sizeof(mpTips)/sizeof(char*)) )
-        i = 0;
-    return string(_(mpTips[i]));
-}
